@@ -1,177 +1,178 @@
 import ast
 import codecs
+import dataclasses
+import logging
 import os
 import sys
 import time
-import traceback
 from difflib import unified_diff
 from typing import Collection, List, Optional, Tuple
 
 import astor
 
-from flynt import state
 from flynt.cli_messages import farewell_message
 from flynt.process import (
     fstringify_code_by_line,
     fstringify_concats,
     fstringify_static_joins,
 )
+from flynt.state import State
+
+log = logging.getLogger(__name__)
 
 blacklist = {".tox", "venv", "site-packages", ".eggs"}
 
 
+@dataclasses.dataclass(frozen=True)
+class FstringifyResult:
+    n_changes: int
+    original_length: int
+    new_length: int
+    content: str
+
+
 def _fstringify_file(
     filename: str,
-    multiline: bool,
-    len_limit: int,
-    *,
-    transform_concat: bool = False,
-    transform_format: bool = True,
-    transform_join: bool = False,
-    transform_percent: bool = True,
-) -> Tuple[bool, int, int, int]:
+    state: State,
+) -> Optional[FstringifyResult]:
     """
-    :return: tuple: (changes_made, n_changes,
-    length of original code, length of new code)
+    F-stringify a file, write changes, and return a change result.
     """
-
-    def default_result():
-        return False, 0, len(contents), len(contents)
-
     encoding, bom = encoding_by_bom(filename)
 
     with open(filename, encoding=encoding, newline="") as f:
         try:
             contents = f.read()
-        except UnicodeDecodeError as e:
-            contents = ""
-            print(f"Exception while reading {filename}: {e}")
-            return default_result()
+        except UnicodeDecodeError:
+            log.error(f"Exception while reading {filename}", exc_info=True)
+            return None
 
+    result = fstringify_content(
+        contents=contents,
+        state=state,
+        filename=filename,
+    )
+
+    if result is None:
+        return None
+
+    new_code = result.content
+    if state.dry_run and result.n_changes:
+        diff = unified_diff(
+            contents.split("\n"),
+            new_code.split("\n"),
+            fromfile=filename,
+        )
+        print("\n".join(diff))
+    elif state.stdout:
+        print(new_code)
+    elif result.n_changes:
+        with open(filename, "wb") as outf:
+            if bom is not None:
+                outf.write(bom)
+            outf.write(new_code.encode(encoding))
+    return result
+
+
+def fstringify_content(
+    contents: str,
+    state: State,
+    filename: str = "<code>",
+) -> Optional[FstringifyResult]:
     try:
         ast_before = ast.parse(contents)
     except SyntaxError:
-        if state.verbose:
-            traceback.print_exc()
-        if not state.quiet:
-            print(f"Can't parse {filename} as a python file.")
-        return default_result()
+        log.exception(f"Can't parse {filename} as a python file.")
+        return None
 
     try:
         new_code = contents
         changes = 0
-        if transform_percent or transform_format:
+        if state.transform_percent or state.transform_format:
             new_code, changes = fstringify_code_by_line(
                 contents,
-                multiline=multiline,
-                len_limit=len_limit,
-                transform_percent=transform_percent,
-                transform_format=transform_format,
+                state=state,
             )
-        if transform_concat:
+        if state.transform_concat:
             new_code, concat_changes = fstringify_concats(
-                new_code, multiline=multiline, len_limit=len_limit
+                new_code,
+                state=state,
             )
             changes += concat_changes
             state.concat_changes += concat_changes
-        if transform_join:
+        if state.transform_join:
             new_code, join_changes = fstringify_static_joins(
-                new_code, multiline=multiline, len_limit=len_limit
+                new_code,
+                state=state,
             )
             changes += join_changes
             state.join_changes += join_changes
 
     except Exception as e:
-        if not state.quiet:
-            if str(e):
-                msg = str(e)
-            else:
-                msg = e.__class__.__name__
-            print(f"Skipping fstrings transform of file {filename} due to {msg}.")
-            if state.verbose:
-                traceback.print_exc()
+        msg = str(e) or e.__class__.__name__
+        log.warning(
+            f"Skipping fstrings transform of file {filename} due to {msg}.",
+            exc_info=True,
+        )
+        return None
 
-        return default_result()
+    result = FstringifyResult(
+        n_changes=changes,
+        original_length=len(contents),
+        new_length=len(new_code),
+        content=new_code,
+    )
 
-    if new_code == contents:
-        return default_result()
+    if result.content == contents:
+        return result
 
     try:
         ast_after = ast.parse(new_code)
     except SyntaxError:
-        print(f"Faulty result during conversion on {filename} - skipping.")
-        if state.verbose:
-            print(new_code)
-            traceback.print_exc()
-        return default_result()
+        log.warning(
+            f"Faulty result during conversion on {filename} - skipping.",
+            exc_info=True,
+        )
+        return None
 
     if not len(ast_before.body) == len(ast_after.body):
-        print(
+        log.error(
             f"Faulty result during conversion on {filename}: "
-            f"statement count has changed, which is not intended - skipping."
+            f"statement count has changed, which is not intended - skipping.",
         )
-        return default_result()
-
-    if state.dry_run:
-        diff = unified_diff(
-            contents.split("\n"), new_code.split("\n"), fromfile=filename
-        )
-        print("\n".join(diff))
-    else:
-        mode = "w"
-        if bom is not None:
-            with open(filename, "wb") as f:
-                f.write(bom)
-            mode = "a"
-        with open(filename, mode, encoding=encoding, newline="") as f:
-            f.write(new_code)
-
-    return True, changes, len(contents), len(new_code)
+        return None
+    return result
 
 
 def fstringify_files(
-    files,
-    multiline,
-    len_limit,
-    transform_concat: bool = False,
-    transform_join: bool = False,
-    *,
-    transform_format: bool = True,
-    transform_percent: bool = True,
-):
+    files: List[str],
+    state: State,
+) -> int:
     changed_files = 0
     total_charcount_original = 0
     total_charcount_new = 0
     total_expressions = 0
     start_time = time.time()
     for path in files:
-        (
-            changed,
-            count_expressions,
-            charcount_original,
-            charcount_new,
-        ) = _fstringify_file(
+        result = _fstringify_file(
             path,
-            multiline,
-            len_limit,
-            transform_concat=transform_concat,
-            transform_format=transform_format,
-            transform_join=transform_join,
-            transform_percent=transform_percent,
+            state,
         )
-        if changed:
-            changed_files += 1
-            total_expressions += count_expressions
-        total_charcount_original += charcount_original
-        total_charcount_new += charcount_new
-
-        if state.verbose:
-            status = "modified" if count_expressions else "no change"
-            print(f"fstringifying {path}...{status}")
+        if result:
+            if result.n_changes:
+                changed_files += 1
+                total_expressions += result.n_changes
+            total_charcount_original += result.original_length
+            total_charcount_new += result.n_changes
+            status = "modified" if result.n_changes else "no change"
+        else:
+            status = "failed"
+        log.info(f"fstringifying {path}...{status}")
     total_time = time.time() - start_time
 
     if not state.quiet:
         _print_report(
+            state,
             len(files),
             changed_files,
             total_charcount_new,
@@ -184,8 +185,14 @@ def fstringify_files(
 
 
 def _print_report(
-    found_files, changed_files, total_cc_new, total_cc_original, total_expr, total_time
-):
+    state: State,
+    found_files: int,
+    changed_files: int,
+    total_cc_new: int,
+    total_cc_original: int,
+    total_expr: int,
+    total_time: float,
+) -> None:
     print("\nFlynt run has finished. Stats:")
     print(f"\nExecution time:                            {total_time:.3f}s")
     print(f"Files checked:                             {found_files}")
@@ -194,7 +201,7 @@ def _print_report(
         cc_reduction = total_cc_original - total_cc_new
         cc_percent_reduction = cc_reduction / total_cc_original
         print(
-            f"Character count reduction:                 {cc_reduction} ({cc_percent_reduction:.2%})\n"
+            f"Character count reduction:                 {cc_reduction} ({cc_percent_reduction:.2%})\n",
         )
 
         print("Per expression type:")
@@ -202,7 +209,7 @@ def _print_report(
             percent_fraction = state.percent_transforms / state.percent_candidates
             print(
                 f"Old style (`%`) expressions attempted:     {state.percent_transforms}/"
-                f"{state.percent_candidates} ({percent_fraction:.1%})"
+                f"{state.percent_candidates} ({percent_fraction:.1%})",
             )
         else:
             print("No old style (`%`) expressions attempted.")
@@ -210,7 +217,7 @@ def _print_report(
         if state.call_candidates:
             print(
                 f"`.format(...)` calls attempted:            {state.call_transforms}/"
-                f"{state.call_candidates} ({state.call_transforms / state.call_candidates:.1%})"
+                f"{state.call_candidates} ({state.call_transforms / state.call_candidates:.1%})",
             )
         else:
             print("No `.format(...)` calls attempted.")
@@ -218,7 +225,7 @@ def _print_report(
         if state.concat_candidates:
             print(
                 f"String concatenations attempted:           {state.concat_changes}/"
-                f"{state.concat_candidates} ({state.concat_changes / state.concat_candidates:.1%})"
+                f"{state.concat_candidates} ({state.concat_changes / state.concat_candidates:.1%})",
             )
         else:
             print("No concatenations attempted.")
@@ -226,7 +233,7 @@ def _print_report(
         if state.join_candidates:
             print(
                 f"Static string joins attempted:             {state.join_changes}/"
-                f"{state.join_candidates} ({state.join_changes / state.join_candidates:.1%})"
+                f"{state.join_candidates} ({state.join_changes / state.join_candidates:.1%})",
             )
         else:
             print("No static string joins attempted.")
@@ -235,7 +242,7 @@ def _print_report(
 
         if state.invalid_conversions:
             print(
-                f"Out of all attempted transforms, {state.invalid_conversions} resulted in errors."
+                f"Out of all attempted transforms, {state.invalid_conversions} resulted in errors.",
             )
             print("To find out specific error messages, use --verbose flag.")
 
@@ -245,40 +252,28 @@ def _print_report(
 
 def fstringify(
     files_or_paths: List[str],
-    multiline: bool,
-    len_limit: int,
+    state: State,
     fail_on_changes: bool = False,
-    transform_percent: bool = True,
-    transform_format: bool = True,
-    transform_concat: bool = False,
-    transform_join: bool = False,
     excluded_files_or_paths: Optional[Collection[str]] = None,
 ) -> int:
     """determine if a directory or a single file was passed, and f-stringify it."""
-
     files = _resolve_files(files_or_paths, excluded_files_or_paths)
 
     status = fstringify_files(
         files,
-        multiline=multiline,
-        len_limit=len_limit,
-        transform_percent=transform_percent,
-        transform_format=transform_format,
-        transform_concat=transform_concat,
-        transform_join=transform_join,
+        state=state,
     )
 
     if fail_on_changes:
         return status
-    else:
-        return 0
+    return 0
 
 
 def _resolve_files(
-    files_or_paths: List[str], excluded_files_or_paths: Optional[Collection[str]]
+    files_or_paths: List[str],
+    excluded_files_or_paths: Optional[Collection[str]],
 ) -> List[str]:
     """Resolve relative paths and directory names into a list of absolute paths to python files."""
-
     files = []
     _blacklist = blacklist.copy()
     if excluded_files_or_paths is not None:
@@ -302,9 +297,8 @@ def _resolve_files(
     return files
 
 
-def encoding_by_bom(path, default="utf-8") -> Tuple[str, Optional[bytes]]:
+def encoding_by_bom(path: str, default: str = "utf-8") -> Tuple[str, Optional[bytes]]:
     """Adapted from https://stackoverflow.com/questions/13590749/reading-unicode-file-data-with-bom-chars-in-python/24370596#24370596"""
-
     with open(path, "rb") as f:
         raw = f.read(4)  # will read less if the file is smaller
     # BOM_UTF32_LE's start is equal to BOM_UTF16_LE so need to try the former first
