@@ -1,19 +1,23 @@
 """This module parses the command line arguments and passes them to flynt.api.fstringify."""
 
 import argparse
+import logging
+import os
 import sys
 import warnings
+from typing import List, Optional
 
-from flynt import __version__, state
-from flynt.api import fstringify, fstringify_code_by_line
+from flynt import __version__
+from flynt.api import fstringify, fstringify_content
 from flynt.pyproject_finder import find_pyproject_toml, parse_pyproject_toml
+from flynt.state import State
 
 
 def main():
     return sys.exit(run_flynt_cli())
 
 
-def run_flynt_cli(arglist=None):
+def run_flynt_cli(arglist: Optional[List[str]] = None) -> int:
     """"""
     parser = argparse.ArgumentParser(
         prog="flynt",
@@ -33,7 +37,7 @@ def run_flynt_cli(arglist=None):
         "-q",
         "--quiet",
         action="store_true",
-        help="run without stdout output",
+        help="run without outputting statistics to stdout",
         default=False,
     )
 
@@ -54,7 +58,8 @@ def run_flynt_cli(arglist=None):
         default=88,
     )
 
-    parser.add_argument(
+    group = parser.add_mutually_exclusive_group()
+    group.add_argument(
         "-d",
         "--dry-run",
         action="store_true",
@@ -64,6 +69,15 @@ def run_flynt_cli(arglist=None):
         "used for linting purposes.",
     )
 
+    group.add_argument(
+        "--stdout",
+        action="store_true",
+        default=False,
+        help="Do not change the files in-place and print the result instead. "
+        "This argument implies --quiet, i.e. no statistics are printed to stdout, "
+        "only the resulting code. It is incompatible with --dry-run and --verbose.",
+    )
+
     parser.add_argument(
         "-s",
         "--string",
@@ -134,7 +148,13 @@ def run_flynt_cli(arglist=None):
     )
 
     parser.add_argument(
-        "src", action="store", nargs="*", help="source file(s) or directory"
+        "src",
+        action="store",
+        nargs="*",
+        help=(
+            "source file(s) or directory "
+            "(or a single `-` to read stdin and output to stdout)"
+        ),
     )
 
     parser.add_argument(
@@ -144,11 +164,13 @@ def run_flynt_cli(arglist=None):
         help="Print the current version number and exit.",
     )
     args = parser.parse_args(arglist)
+    if args.stdout and args.verbose:
+        parser.error("--stdout should not be used with -v/--verbose")
 
     if args.version:
         print(__version__)
         return 0
-    elif not args.src:
+    if not args.src:
         print("flynt: error: the following arguments are required: src")
         parser.print_usage()
         return 1
@@ -156,25 +178,44 @@ def run_flynt_cli(arglist=None):
     if args.transform_concats and sys.version_info < (3, 8):
         raise Exception(
             f"""Transforming string concatenations is only possible with flynt
-                installed to a python3.8+ interpreter. Currently using {sys.version_info}."""
+                installed to a python3.8+ interpreter. Currently using {sys.version_info}.""",
         )
 
     if args.transform_joins and sys.version_info < (3, 8):
         raise Exception(
             f"""Transforming joins is only possible with flynt
-                installed to a python3.8+ interpreter. Currently using {sys.version_info}."""
+                installed to a python3.8+ interpreter. Currently using {sys.version_info}.""",
         )
 
+    logging.basicConfig(
+        format="%(message)s",
+        level=(logging.DEBUG if args.verbose else logging.CRITICAL),
+    )
+
+    state = state_from_args(args)
+    if args.verbose:
+        logging.getLogger("flynt").setLevel(logging.DEBUG)
+
     if args.string:
-        set_global_state(args)
-        converted, _ = fstringify_code_by_line(
-            " ".join(args.src),
-            multiline=not args.no_multiline,
-            len_limit=int(args.line_length),
+        content = " ".join(args.src)
+        result = fstringify_content(
+            content,
+            state,
         )
-        print(converted)
+        print(result.content if result else content)
+        return 0
+    if "-" in args.src:
+        if len(args.src) > 1:
+            parser.error("Cannot use '-' with a list of other paths")
+        result = fstringify_content(
+            sys.stdin.read()[: -len(os.linesep)],
+            state,
+            filename="<stdin>",
+        )
+        if not result:
+            return 1
+        print(result.content)
         return 0
-
     salutation = f"Running flynt v.{__version__}"
     toml_file = find_pyproject_toml(tuple(args.src))
     if toml_file:
@@ -187,12 +228,12 @@ def run_flynt_cli(arglist=None):
             warnings.warn(
                 f"Unknown config options: {redundant}. "
                 f"This might be a spelling problem. "
-                f"Supported options are: {supported_args}"
+                f"Supported options are: {supported_args}",
             )
         parser.set_defaults(**cfg)
         args = parser.parse_args(arglist)
-    set_global_state(args)
-    if not args.quiet:
+        state = state_from_args(args)
+    if not state.quiet:
         print(salutation)
     if args.verbose:
         print(f"Using following options: {args}")
@@ -201,18 +242,21 @@ def run_flynt_cli(arglist=None):
     return fstringify(
         args.src,
         excluded_files_or_paths=args.exclude,
-        multiline=not args.no_multiline,
-        len_limit=int(args.line_length),
         fail_on_changes=args.fail_on_change,
-        transform_percent=args.transform_percent,
-        transform_format=args.transform_format,
-        transform_concat=args.transform_concats,
-        transform_join=args.transform_joins,
+        state=state,
     )
 
 
-def set_global_state(args):
-    state.aggressive = args.aggressive
-    state.verbose = args.verbose
-    state.quiet = args.quiet
-    state.dry_run = args.dry_run
+def state_from_args(args) -> State:
+    return State(
+        aggressive=args.aggressive,
+        dry_run=args.dry_run,
+        stdout=args.stdout,
+        len_limit=args.line_length,
+        multiline=(not args.no_multiline),
+        quiet=args.quiet or args.stdout,
+        transform_concat=args.transform_concats,
+        transform_format=args.transform_format,
+        transform_join=args.transform_joins,
+        transform_percent=args.transform_percent,
+    )
