@@ -1,14 +1,17 @@
-import math
+import logging
 import re
 import string
+import sys
 from functools import partial
-from typing import Callable, Tuple
+from typing import Callable, List, Optional, Tuple, Union
 
-from flynt import lexer, state
+from flynt.ast_chunk import AstChunk
 from flynt.exceptions import FlyntException
 from flynt.format import QuoteTypes as qt
 from flynt.format import get_quote_type
 from flynt.lexer import split
+from flynt.lexer.Chunk import Chunk
+from flynt.state import State
 from flynt.static_join.candidates import join_candidates
 from flynt.static_join.transformer import transform_join
 from flynt.string_concat.candidates import concat_candidates
@@ -17,6 +20,8 @@ from flynt.transform.transform import transform_chunk
 
 noqa_regex = re.compile("#[ ]*noqa.*flynt")
 
+log = logging.getLogger(__name__)
+
 
 class JoinTransformer:
     """JoinTransformer fills up the resulting code by tracking
@@ -26,27 +31,26 @@ class JoinTransformer:
     def __init__(
         self,
         code: str,
-        len_limit: int,
+        len_limit: Optional[int],
         candidates_iter_factory: Callable,
         transform_func: Callable,
-    ):
-
+    ) -> None:
         if len_limit is None:
-            len_limit = math.inf
+            len_limit = sys.maxsize
 
         self.len_limit = len_limit
         self.candidates_iter = candidates_iter_factory(code)
         self.transform_func = transform_func
         self.src_lines = code.split("\n")
 
-        self.results = []
+        self.results: List[str] = []
         self.count_expressions = 0
 
         self.last_line = 0
         self.last_idx = 0
         self.used_up = False
 
-    def fstringify_code_by_line(self):
+    def fstringify_code_by_line(self) -> Tuple[str, int]:
         assert not self.used_up, "Tried to use JT twice."
         for chunk in self.candidates_iter:
             self.fill_up_to(chunk)
@@ -56,11 +60,11 @@ class JoinTransformer:
         self.used_up = True
         return "".join(self.results)[:-1], self.count_expressions
 
-    def fill_up_to(self, chunk):
+    def fill_up_to(self, chunk: Union[Chunk, AstChunk]) -> None:
         start_line, start_idx, _ = (chunk.start_line, chunk.start_idx, chunk.end_idx)
         if start_line == self.last_line:
             self.results.append(
-                self.src_lines[self.last_line][self.last_idx : start_idx]
+                self.src_lines[self.last_line][self.last_idx : start_idx],
             )
         else:
             self.results.append(self.src_lines[self.last_line][self.last_idx :] + "\n")
@@ -72,12 +76,12 @@ class JoinTransformer:
 
         self.last_idx = start_idx
 
-    def fill_up_to_line(self, start_line):
+    def fill_up_to_line(self, start_line: int) -> None:
         while self.last_line < start_line:
             self.results.append(self.src_lines[self.last_line] + "\n")
             self.last_line += 1
 
-    def try_chunk(self, chunk):
+    def try_chunk(self, chunk: Union[Chunk, AstChunk]) -> None:
 
         for line in self.src_lines[chunk.start_line : chunk.start_line + chunk.n_lines]:
             if noqa_regex.findall(line):
@@ -104,7 +108,13 @@ class JoinTransformer:
                 rest = next_line[chunk.end_idx :]
             self.maybe_replace(chunk, contract_lines, converted, rest)
 
-    def maybe_replace(self, chunk, contract_lines, converted, rest):
+    def maybe_replace(
+        self,
+        chunk: Union[Chunk, AstChunk],
+        contract_lines: int,
+        converted: str,
+        rest: str,
+    ) -> None:
         if contract_lines:
             if get_quote_type(str(chunk)) in (qt.triple_double, qt.triple_single):
                 lines = converted.split("\\n")
@@ -121,12 +131,12 @@ class JoinTransformer:
         else:
             lines_fit = True
         if contract_lines and not lines_fit:
-            if state.verbose:
-                print(
-                    "Skipping conversion due to line length limit. "
-                    "Pass -ll 999 to increase it. "
-                    "(999 is an example, as number of characters.)"
-                )
+            log.warning(
+                "Skipping conversion of %s due to line length limit. "
+                "Pass -ll 999 to increase it. "
+                "(999 is an example, as number of characters.)",
+                str(chunk),
+            )
             return
 
         self.results.append(converted)
@@ -137,7 +147,8 @@ class JoinTransformer:
         # remove redundant parenthesis
         if len(self.results) < 2 or not self.results[-2]:
             return
-        elif len(self.src_lines[self.last_line]) == self.last_idx:
+
+        if len(self.src_lines[self.last_line]) == self.last_idx:
             return
 
         if (
@@ -147,15 +158,14 @@ class JoinTransformer:
             for char in reversed(self.results[-2][:-1]):
                 if char in string.whitespace:
                     continue
-                elif char in "(=[+*":
+                if char in "(=[+*":
                     break
-                else:
-                    return
+                return
 
             self.results[-2] = self.results[-2][:-1]
             self.last_idx += 1
 
-    def add_rest(self):
+    def add_rest(self) -> None:
         self.results.append(self.src_lines[self.last_line][self.last_idx :] + "\n")
         self.last_line += 1
 
@@ -164,50 +174,46 @@ class JoinTransformer:
             self.last_line += 1
 
 
-def fstringify_code_by_line(
-    code: str,
-    multiline=True,
-    len_limit=88,
-    transform_percent: bool = True,
-    transform_format: bool = True,
-) -> Tuple[str, int]:
+def fstringify_code_by_line(code: str, state: State) -> Tuple[str, int]:
     """returns fstringified version of the code and amount of lines edited."""
-    phunk = partial(
-        transform_chunk,
-        transform_format=transform_format,
-        transform_percent=transform_percent,
-    )
     return _transform_code(
-        code, split.get_fstringify_chunks, phunk, multiline, len_limit
+        code,
+        partial(split.get_fstringify_chunks, lexer_context=state.lexer_context),
+        partial(transform_chunk, state=state),
+        state,
     )
 
 
-def fstringify_concats(code: str, multiline=True, len_limit=88) -> Tuple[str, int]:
+def fstringify_concats(code: str, state: State) -> Tuple[str, int]:
     """replace string literal concatenations with f-string expressions."""
     return _transform_code(
-        code, concat_candidates, transform_concat, multiline, len_limit
+        code,
+        partial(concat_candidates, state=state),
+        partial(transform_concat, state=state),
+        state,
     )
 
 
-def fstringify_static_joins(code: str, multiline=True, len_limit=88) -> Tuple[str, int]:
+def fstringify_static_joins(code: str, state: State) -> Tuple[str, int]:
     """replace joins on static content with f-string expressions."""
-    return _transform_code(code, join_candidates, transform_join, multiline, len_limit)
+    return _transform_code(
+        code,
+        partial(join_candidates, state=state),
+        partial(transform_join, state=state),
+        state,
+    )
 
 
 def _transform_code(
-    code: str, candidates_iter_factory, transform_func, multiline, len_limit
+    code: str,
+    candidates_iter_factory: Callable,
+    transform_func: Callable,
+    state: State,
 ) -> Tuple[str, int]:
     """returns fstringified version of the code and amount of lines edited."""
-
-    len_limit = _multiline_settings(len_limit, multiline)
-    jt = JoinTransformer(code, len_limit, candidates_iter_factory, transform_func)
-    return jt.fstringify_code_by_line()
-
-
-def _multiline_settings(len_limit, multiline):
-    if not multiline:
-        len_limit = 0
-        lexer.set_single_line()
-    else:
-        lexer.set_multiline()
-    return len_limit
+    return JoinTransformer(
+        code,
+        state.len_limit,
+        candidates_iter_factory,
+        transform_func,
+    ).fstringify_code_by_line()
