@@ -1,17 +1,17 @@
 import ast
 from typing import List
 
-from flynt import state
 from flynt.ast_chunk import AstChunk
+from flynt.state import State
 from flynt.static_join.utils import get_static_join_bits
 
 
 class JoinHound(ast.NodeVisitor):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.victims: List[AstChunk] = []
 
-    def visit_Call(self, node: ast.Call):
+    def visit_Call(self, node: ast.Call) -> None:
         """
         Finds all nodes that are joins with a static string literal
         as the joiner and a static iterable as the joinee.
@@ -22,7 +22,7 @@ class JoinHound(ast.NodeVisitor):
             self.generic_visit(node)
 
 
-def join_candidates(code: str):
+def join_candidates(code: str, state: State):
     tree = ast.parse(code)
 
     ch = JoinHound()
