@@ -1,5 +1,5 @@
 import ast
-from typing import Tuple
+from typing import List, Tuple
 
 from flynt.static_join.utils import get_static_join_bits
 from flynt.utils import ast_formatted_value, ast_string_node, fixup_transformed
@@ -19,7 +19,7 @@ class JoinTransformer(ast.NodeTransformer):
             return self.generic_visit(node)
         joiner, args = res
         self.counter += 1
-        args_with_interleaved_joiner = []
+        args_with_interleaved_joiner: List[ast.AST] = []
         for arg in args:
             if isinstance(arg, ast.Str):
                 args_with_interleaved_joiner.append(arg)
@@ -28,7 +28,14 @@ class JoinTransformer(ast.NodeTransformer):
             args_with_interleaved_joiner.append(ast_string_node(joiner))
         args_with_interleaved_joiner.pop()  # remove the last joiner
         if all(isinstance(arg, ast.Str) for arg in args_with_interleaved_joiner):
-            return ast.Str(s="".join(arg.s for arg in args_with_interleaved_joiner))
+            # Additional `isinstance` required to satisfy mypy
+            return ast.Str(
+                s="".join(
+                    arg.s
+                    for arg in args_with_interleaved_joiner
+                    if isinstance(arg, ast.Str)
+                ),
+            )
         return ast.JoinedStr(args_with_interleaved_joiner)
 
 
