@@ -1,13 +1,13 @@
 import ast
-from typing import List
+from typing import Generator, List
 
-from flynt import state
 from flynt.ast_chunk import AstChunk
+from flynt.state import State
 from flynt.utils import is_str_literal
 
 
-def is_string_concat(node):
-    """Returns True for nodes representing a string concatenation"""
+def is_string_concat(node: ast.AST) -> bool:
+    """Returns True for nodes representing a string concatenation."""
     if is_str_literal(node):
         return True
     if isinstance(node, ast.BinOp) and isinstance(node.op, ast.Add):
@@ -16,13 +16,13 @@ def is_string_concat(node):
 
 
 class ConcatHound(ast.NodeVisitor):
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.victims: List[AstChunk] = []
 
-    def visit_BinOp(self, node: ast.BinOp):
+    def visit_BinOp(self, node: ast.BinOp) -> None:
         """
-        Finds all nodes that are string concatenations with a literal
+        Finds all nodes that are string concatenations with a literal.
         """
         if is_string_concat(node):
             self.victims.append(AstChunk(node))
@@ -30,7 +30,7 @@ class ConcatHound(ast.NodeVisitor):
             self.generic_visit(node)
 
 
-def concat_candidates(code: str):
+def concat_candidates(code: str, state: State) -> Generator[AstChunk, None, None]:
     tree = ast.parse(code)
 
     ch = ConcatHound()
