@@ -2,41 +2,41 @@ import ast
 import string
 import sys
 from collections import deque
-from typing import Tuple, Union
+from typing import Any, Dict, List, Tuple, Union
 
-from flynt import state
 from flynt.exceptions import ConversionRefused, FlyntException
 from flynt.utils import ast_formatted_value, ast_string_node
 
 
-def matching_call(node) -> bool:
+def matching_call(node: ast.Call) -> bool:
     """
     Check if an ast Node represents a "...".format() call.
     """
-    call_from_string = (
-        isinstance(node.func, ast.Attribute)
-        and hasattr(node.func, "value")
-        and (
-            isinstance(node.func.value, ast.Str)
-            or (
-                isinstance(node.func.value, ast.Constant)
-                and isinstance(node.func.value.value, str)
-            )
-        )
-    )
-    return (
-        isinstance(node, ast.Call) and call_from_string and node.func.attr == "format"
+    func = node.func
+    if not isinstance(func, ast.Attribute):
+        return False
+    call_from_string = hasattr(func, "value") and (
+        isinstance(func.value, ast.Str)
+        or (isinstance(func.value, ast.Constant) and isinstance(func.value.value, str))
     )
+    return isinstance(node, ast.Call) and call_from_string and func.attr == "format"
 
 
 stdlib_parse = string.Formatter().parse
 
 
-def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bool]:
+def joined_string(
+    fmt_call: ast.Call,
+    *,
+    aggressive: bool = False,
+) -> Tuple[Union[ast.JoinedStr, ast.Str], bool]:
     """Transform a "...".format() call node into a f-string node."""
-
+    assert isinstance(fmt_call.func, ast.Attribute) and isinstance(
+        fmt_call.func.value,
+        ast.Str,
+    )
     string = fmt_call.func.value.s
-    var_map = {kw.arg: kw.value for kw in fmt_call.keywords}
+    var_map: Dict[Any, Any] = {kw.arg: kw.value for kw in fmt_call.keywords}
     inserted_value_nodes = []
     for a in fmt_call.args:
         inserted_value_nodes += list(ast.walk(a))
@@ -52,7 +52,7 @@ def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bo
     splits = deque(stdlib_parse(string))
 
     seq_ctr = 0
-    new_segments = []
+    new_segments: List[ast.AST] = []
     manual_field_ordering = False
 
     for raw, var_name, fmt_str, conversion in splits:
@@ -64,7 +64,7 @@ def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bo
 
         if "[" in var_name:
             raise FlyntException(
-                f"Skipping f-stringify of a fmt call with indexed name {var_name}"
+                f"Skipping f-stringify of a fmt call with indexed name {var_name}",
             )
 
         suffix = ""
@@ -72,6 +72,7 @@ def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bo
             idx = var_name.find(".")
             var_name, suffix = var_name[:idx], var_name[idx + 1 :]
 
+        identifier: Union[str, int]
         if var_name.isdigit():
             manual_field_ordering = True
             identifier = int(var_name)
@@ -82,7 +83,7 @@ def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bo
         else:
             identifier = var_name
 
-        if state.aggressive:
+        if aggressive:
             ast_name = var_map[identifier]
         else:
             try:
@@ -90,29 +91,27 @@ def joined_string(fmt_call: ast.Call) -> Tuple[Union[ast.JoinedStr, ast.Str], bo
                 ast_name = var_map.pop(identifier)
             except KeyError as e:
                 raise ConversionRefused(
-                    f"A variable {identifier} is used multiple times - better not to replace it."
+                    f"A variable {identifier} is used multiple times - better not to replace it.",
                 ) from e
 
         if suffix:
             ast_name = ast.Attribute(value=ast_name, attr=suffix)
         new_segments.append(ast_formatted_value(ast_name, fmt_str, conversion))
 
-    if var_map and not state.aggressive:
+    if var_map and not aggressive:
         raise FlyntException(
-            f"Some variables were never used: {var_map} - skipping conversion, it's a risk of bug."
+            f"Some variables were never used: {var_map} - skipping conversion, it's a risk of bug.",
         )
 
     def is_literal_string(node):
         if sys.version_info < (3, 8):
             return isinstance(node, ast.Str)
-        else:
-            return isinstance(node, ast.Constant) and isinstance(node.value, str)
+        return isinstance(node, ast.Constant) and isinstance(node.value, str)
 
     def literal_string_value(node):
         if sys.version_info < (3, 8):
             return node.s
-        else:
-            return node.value
+        return node.value
 
     def fix_literals(segment):
         if (
